@@ -57,18 +57,24 @@ ODE interface to OU library functions.
 
 
 using _OU_NAMESPACE::CEnumUnsortedElementArray;
+using _OU_NAMESPACE::CEnumSortedElementArray;
 
 #if dATOMICS_ENABLED
 using _OU_NAMESPACE::atomicord32;
 using _OU_NAMESPACE::atomicptr;
 using _OU_NAMESPACE::InitializeAtomicAPI;
 using _OU_NAMESPACE::FinalizeAtomicAPI;
+using _OU_NAMESPACE::AtomicIncrement;
+using _OU_NAMESPACE::AtomicDecrement;
 using _OU_NAMESPACE::AtomicCompareExchange;
 using _OU_NAMESPACE::AtomicExchange;
 using _OU_NAMESPACE::AtomicExchangeAddNoResult;
 using _OU_NAMESPACE::AtomicExchangeAdd;
 using _OU_NAMESPACE::AtomicCompareExchangePointer;
 using _OU_NAMESPACE::AtomicExchangePointer;
+using _OU_NAMESPACE::AtomicReadReorderBarrier;
+using _OU_NAMESPACE::AtomicStore;
+using _OU_NAMESPACE::AtomicStorePointer;
 #endif
 
 
@@ -85,10 +91,13 @@ public:
 };
 
 
-#else // !dOU_ENABLED
+#endif 
+
+
+#if !dOU_ENABLED || !dATOMICS_ENABLED
 
 typedef unsigned int atomicord32;
-typedef size_t atomicptr;
+typedef void *atomicptr;
 
 
 #endif // dOU_ENABLED
