@@ -36,7 +36,6 @@ ODE Thread Local Storage access stub implementation.
 #include "odemath.h"
 #include "odetls.h"
 #include "collision_trimesh_internal.h"
-#include "util.h"
 
 
 #if dTLS_ENABLED
@@ -101,7 +100,7 @@ void COdeTls::CleanupForThread()
 
 bool COdeTls::AssignDataAllocationFlags(EODETLSKIND tkTLSKind, unsigned uInitializationFlags)
 {
-    bool bResult = CThreadLocalStorage::SetStorageValue(m_ahtkStorageKeys[tkTLSKind], OTI_DATA_ALLOCATION_FLAGS, (tlsvaluetype)(size_t)uInitializationFlags);
+    bool bResult = CThreadLocalStorage::SetStorageValue(m_ahtkStorageKeys[tkTLSKind], OTI_DATA_ALLOCATION_FLAGS, (tlsvaluetype)(sizeint)uInitializationFlags);
     return bResult;
 }
 
@@ -118,7 +117,7 @@ void COdeTls::DestroyTrimeshCollidersCache(EODETLSKIND tkTLSKind)
 {
     TrimeshCollidersCache *pccCacheInstance = (TrimeshCollidersCache *)CThreadLocalStorage::GetStorageValue(m_ahtkStorageKeys[tkTLSKind], OTI_TRIMESH_TRIMESH_COLLIDER_CACHE);
 
-    if (pccCacheInstance)
+    if (pccCacheInstance != NULL)
     {
         FreeTrimeshCollidersCache(pccCacheInstance);
 
@@ -132,7 +131,11 @@ void COdeTls::DestroyTrimeshCollidersCache(EODETLSKIND tkTLSKind)
 
 void COdeTls::FreeTrimeshCollidersCache(TrimeshCollidersCache *pccCacheInstance)
 {
+#if dTRIMESH_ENABLED 
     delete pccCacheInstance;
+#else
+    dIASSERT(pccCacheInstance == NULL); // The cache is not being allocated if the library is configured without trimeshes
+#endif
 }
 
 
