@@ -93,6 +93,10 @@
  *	only.
  */
 
+#include <ou/features.h>
+
+
+#if _OU_FEATURE_SET >= _OU_FEATURE_SET_ATOMICS
 
 #include <ou/inttypes.h>
 #include <ou/namespace.h>
@@ -136,7 +140,7 @@
  *	\fn atomicord32 _OU_CONVENTION_API AtomicIncrement(volatile atomicord32 *paoDestination)
  *	\brief Increments the destination and returns its new value.
  *	\param paoDestination A pointer to a variable to be incremented.
- *	\return A value of variable pointer to by \a paoDestination after the increment.
+ *	\return A value of variable pointed to by \a paoDestination after the increment.
  *
  *	The function implements functionality of \c InterlockedIncrement from Win32 API.
  *	It is most commonly used for reference counting.
@@ -147,22 +151,32 @@
  *	\fn atomicord32 _OU_CONVENTION_API AtomicDecrement(volatile atomicord32 *paoDestination)
  *	\brief Decrements the destination and returns its new value.
  *	\param paoDestination A pointer to a variable to be decremented.
- *	\return A value of variable pointer to by \a paoDestination after the decrement.
+ *	\return A value of variable pointed to by \a paoDestination after the decrement.
  *
  *	The function implements functionality of \c InterlockedDecrement from Win32 API.
  *	It is most commonly used for reference counting.
  *	\see AtomicIncrement
  */
 
+/**
+ *	\fn void _OU_CONVENTION_API AtomicStore(volatile atomicord32 *paoDestination, atomicord32 aoValue)
+ *	\brief Stores the value in destination.
+ *	\param paoDestination A pointer to a variable the data is to be stored in.
+ *	\param aoValue A value to be stored. 
+ *
+ *	The function performs atomic store of \a aoValue value in the memory location 
+ *	pointed to by \a paoDestination.
+ */
+
 /**
  *	\fn atomicord32 _OU_CONVENTION_API AtomicExchange(volatile atomicord32 *paoDestination, atomicord32 aoExchange)
  *	\brief Stores new value in destination and returns old value of destination.
  *	\param paoDestination A pointer to a variable the data is to be exchanged with.
  *	\param aoExchange A value to be used for exchange. 
- *	\return Previous value of variable pointer to by \a paoDestination.
+ *	\return Previous value of variable pointed to by \a paoDestination.
  *
  *	The function performs atomic exchange of \a aoExchange value with memory location 
- *	pointer to by \a paoDestination. Most common uses are relaxed synchronization
+ *	pointed to by \a paoDestination. Most common uses are relaxed synchronization
  *	and shared value extraction.
  *	\see AtomicCompareExchange
  */
@@ -173,9 +187,9 @@
  *	and returns original value of destination.
  *	\param paoDestination A pointer to a variable the value is to be added to.
  *	\param aoAddend An addend to be used in operation.
- *	\return Original value of variable pointer to by \a paoDestination.
+ *	\return Original value of variable pointed to by \a paoDestination.
  *
- *	The function computes sum of \a aoAddend and location pointer to by \a paoDestination
+ *	The function computes sum of \a aoAddend and location pointed to by \a paoDestination
  *	stores it in the location and returns original value instead. The function is 
  *	close to both exchange and increment groups by semantics but it has been
  *	put into exchange group because it returns original value of the destination.
@@ -212,7 +226,7 @@
  *
  *	\c AtomicAnd updates variable pointed to by \a paoDestination with result of
  *	 bitwise AND of \a aoBitMask and existing \a paoDestination target. The result 
- *	is original value that was pointer to by \a paoDestination before the operation
+ *	is original value that was pointed to by \a paoDestination before the operation
  *	was performed. Common applications are object state manipulations.
  *	\see AtomicOr
  *	\see AtomicXor
@@ -228,7 +242,7 @@
  *
  *	\c AtomicOr updates variable pointed to by \a paoDestination with result of
  *	 bitwise OR of \a aoBitMask and existing \a paoDestination target. The result 
- *	is original value that was pointer to by \a paoDestination before the operation 
+ *	is original value that was pointed to by \a paoDestination before the operation 
  *	was performed. Common applications are object state manipulations.
  *	\see AtomicAnd
  *	\see AtomicXor
@@ -244,12 +258,19 @@
  *
  *	\c AtomicXor updates variable pointed to by \a paoDestination with result of
  *	 bitwise XOR of \a aoBitMask and existing \a paoDestination target. The result 
- *	is original value that was pointer to by \a paoDestination before the operation 
+ *	is original value that was pointed to by \a paoDestination before the operation 
  *	was performed. Common applications are object state manipulations.
  *	\see AtomicAnd
  *	\see AtomicOr
  */
 
+/**
+ *	\fn void _OU_CONVENTION_API AtomicStorePointer(volatile atomicptr *papDestination, atomicptr apValue)
+ *	\brief The function is identical to \c AtomicStore except that it operates
+ *	with pointers rather than 32-bit integers.
+ *	\see AtomicStore
+ */
+
 /**
  *	\fn atomicptr _OU_CONVENTION_API AtomicExchangePointer(volatile atomicptr *papDestination, atomicptr apExchange)
  *	\brief The function is identical to \c AtomicExchange except that it operates
@@ -324,6 +345,13 @@
  *	\see AtomicXor
  */
 
+/**
+ *	\fn void _OU_CONVENTION_API AtomicReadReorderBarrier()
+ *	\brief The function prevents both the compiler and the processor architecture
+ *  from reordering memory read operations across the barrier call.
+ */
+
+
 /**
  *	\fn bool _OU_CONVENTION_API InitializeAtomicAPI()
  *	\brief Performs initialization tasks to allow using atomic functions.
@@ -374,7 +402,7 @@ END_NAMESPACE_OU();
 BEGIN_NAMESPACE_OU();
 
 
-typedef LONG atomicord32;
+typedef ULONG atomicord32;
 typedef PVOID atomicptr;
 
 
@@ -398,11 +426,11 @@ typedef PVOID atomicptr;
 
 #else // other compilers
 
-#define __ou_intlck_value_t atomicord32
-#define __ou_intlck_target_t volatile atomicord32 *
+#define __ou_intlck_value_t LONG
+#define __ou_intlck_target_t volatile LONG *
 #define __ou_xchgadd_target_t LPLONG
-#define __ou_cmpxchg_value_t atomicord32
-#define __ou_cmpxchg_target_t volatile atomicord32 *
+#define __ou_cmpxchg_value_t LONG
+#define __ou_cmpxchg_target_t volatile LONG *
 
 
 #endif // #if _OU_COMPILER == _OU_COMPILER_GCC
@@ -540,6 +568,27 @@ static _OU_ALWAYSINLINE bool _OU_CONVENTION_API
 }
 
 
+#if _OU_COMPILER == _OU_COMPILER_MSVC
+ 
+#if (_OU_TARGET_ARCH == _OU_TARGET_ARCH_X86 || _OU_TARGET_ARCH == _OU_TARGET_ARCH_X64)
+
+#define __OU_ATOMIC_READREORDERBARRIER_FUNCTION_DEFINED
+
+#include <intrin.h>
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicReadReorderBarrier()
+{
+    _ReadBarrier();  
+}
+
+
+#endif // #if (_OU_TARGET_ARCH == _OU_TARGET_ARCH_X86 || _OU_TARGET_ARCH == _OU_TARGET_ARCH_X64)
+
+
+#endif // #if _OU_COMPILER == _OU_COMPILER_MSVC
+
+
 #undef __ou_intlck_value_t
 #undef __ou_intlck_target_t
 #undef __ou_xchgadd_target_t
@@ -682,10 +731,14 @@ END_NAMESPACE_OU();
 BEGIN_NAMESPACE_OU();
 
 
-typedef int32_t atomicord32;
+typedef uint32_t atomicord32;
 typedef void *atomicptr;
 
 
+#define __ou_intlck_target_t volatile int32_t *
+#define __ou_xchgadd_target_t volatile int32_t *
+#define __ou_cmpxchg_value_t int32_t
+#define __ou_cmpxchg_target_t volatile int32_t *
 #define __ou_bitmsk_target_t volatile uint32_t *
 
 
@@ -694,20 +747,20 @@ typedef void *atomicptr;
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicIncrement(volatile atomicord32 *paoDestination)
 {
-	return OSAtomicIncrement32Barrier(paoDestination);
+	return OSAtomicIncrement32Barrier((__ou_intlck_target_t)paoDestination);
 }
 
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicDecrement(volatile atomicord32 *paoDestination)
 {
-	return OSAtomicDecrement32Barrier(paoDestination);
+	return OSAtomicDecrement32Barrier((__ou_intlck_target_t)paoDestination);
 }
 
 
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicExchange(volatile atomicord32 *paoDestination, atomicord32 aoExchange)
 {
-	atomicord32 aoOldValue = *paoDestination;
+	__ou_cmpxchg_value_t aoOldValue = *paoDestination;
 
 	/*
 	 *	Implementation Note:
@@ -717,8 +770,8 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 	 *	threads will be using this API set for manipulations with paoDestination as well
 	 *	and hence will not issue writes after/without memory barrier.
 	 */
-	for (bool bSwapExecuted = OSAtomicCompareAndSwap32Barrier(aoOldValue, aoExchange, paoDestination);
-		!bSwapExecuted; bSwapExecuted = OSAtomicCompareAndSwap32(aoOldValue, aoExchange, paoDestination))
+	for (bool bSwapExecuted = OSAtomicCompareAndSwap32Barrier(aoOldValue, aoExchange, (__ou_cmpxchg_target_t)paoDestination);
+		!bSwapExecuted; bSwapExecuted = OSAtomicCompareAndSwap32(aoOldValue, aoExchange, (__ou_cmpxchg_target_t)paoDestination))
 	{
 		aoOldValue = *paoDestination;
 	}
@@ -729,13 +782,13 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicExchangeAdd(volatile atomicord32 *paoDestination, atomicord32 aoAddend)
 {
-	return (OSAtomicAdd32Barrier(aoAddend, paoDestination) - aoAddend);
+	return (OSAtomicAdd32Barrier(aoAddend, (__ou_xchgadd_target_t)paoDestination) - aoAddend);
 }
 
 static _OU_ALWAYSINLINE bool _OU_CONVENTION_API 
 /*bool */AtomicCompareExchange(volatile atomicord32 *paoDestination, atomicord32 aoComparand, atomicord32 aoExchange)
 {
-	return OSAtomicCompareAndSwap32Barrier(aoComparand, aoExchange, paoDestination);
+	return OSAtomicCompareAndSwap32Barrier(aoComparand, aoExchange, (__ou_cmpxchg_target_t)paoDestination);
 }
 
 
@@ -861,19 +914,19 @@ static _OU_ALWAYSINLINE bool _OU_CONVENTION_API
 static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
 /*void */AtomicIncrementNoResult(volatile atomicord32 *paoDestination)
 {
-	OSAtomicIncrement32Barrier(paoDestination);
+	OSAtomicIncrement32Barrier((__ou_intlck_target_t)paoDestination);
 }
 
 static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
 /*void */AtomicDecrementNoResult(volatile atomicord32 *paoDestination)
 {
-	OSAtomicDecrement32Barrier(paoDestination);
+	OSAtomicDecrement32Barrier((__ou_intlck_target_t)paoDestination);
 }
 
 static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
 /*void */AtomicExchangeAddNoResult(volatile atomicord32 *paoDestination, atomicord32 aoAddend)
 {
-	OSAtomicAdd32Barrier(aoAddend, paoDestination);
+	OSAtomicAdd32Barrier(aoAddend, (__ou_xchgadd_target_t)paoDestination);
 }
 
 static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
@@ -899,6 +952,11 @@ static _OU_ALWAYSINLINE void _OU_CONVENTION_API
 
 
 #undef __ou_bitmsk_target_t
+#undef __ou_cmpxchg_target_t
+#undef __ou_cmpxchg_value_t
+#undef __ou_xchgadd_target_t
+#undef __ou_intlck_target_t
+
 
 #endif // #if _OU_TARGET_OS == _OU_TARGET_OS_MAC
 
@@ -918,9 +976,12 @@ END_NAMESPACE_OU();
 BEGIN_NAMESPACE_OU();
 
 
-typedef int atomicord32;
+typedef unsigned int atomicord32;
 typedef void *atomicptr;
 
+#define __ou_cas_value_t int
+#define __ou_caslp_value_t long
+
 
 #define __OU_ATOMIC_ORD32_FUNCTIONS_DEFINED
 
@@ -940,7 +1001,7 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicExchange(volatile atomicord32 *paoDestination, atomicord32 aoExchange)
 {
-	atomicord32 aoOldValue = *paoDestination;
+	__ou_cas_value_t aoOldValue = *paoDestination;
 
 	while (!compare_and_swap((atomic_p)paoDestination, &aoOldValue, aoExchange))
 	{
@@ -959,7 +1020,7 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE bool _OU_CONVENTION_API 
 /*bool */AtomicCompareExchange(volatile atomicord32 *paoDestination, atomicord32 aoComparand, atomicord32 aoExchange)
 {
-	atomicord32 aoOldValue = aoComparand;
+	__ou_cas_value_t aoOldValue = aoComparand;
 
 	return compare_and_swap((atomic_p)paoDestination, &aoOldValue, aoExchange);
 }
@@ -982,7 +1043,7 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicXor(volatile atomicord32 *paoDestination, atomicord32 aoBitMask)
 {
-	volatile atomicord32 aoOldValue = *paoDestination;
+	volatile __ou_cas_value_t aoOldValue = *paoDestination;
 	
 	while (!compare_and_swap((atomic_p)paoDestination, &aoOldValue, aoOldValue ^ aoBitMask))
 	{
@@ -1000,9 +1061,9 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicptr _OU_CONVENTION_API 
 /*atomicptr */AtomicExchangePointer(volatile atomicptr *papDestination, atomicptr apExchange)
 {
-	long liOldValue = (long)*papDestination;
+	__ou_caslp_value_t liOldValue = (__ou_caslp_value_t)*papDestination;
 	
-	while (!compare_and_swaplp((atomic_l)papDestination, &liOldValue, (long)apExchange))
+	while (!compare_and_swaplp((atomic_l)papDestination, &liOldValue, (__ou_caslp_value_t)apExchange))
 	{
 		// Do nothing
 	}
@@ -1013,15 +1074,19 @@ static _OU_ALWAYSINLINE atomicptr _OU_CONVENTION_API
 static _OU_ALWAYSINLINE bool _OU_CONVENTION_API 
 /*bool */AtomicCompareExchangePointer(volatile atomicptr *papDestination, atomicptr apComparand, atomicptr apExchange)
 {
-	long liOldValue = (long)apComparand;
+	__ou_caslp_value_t liOldValue = (__ou_caslp_value_t)apComparand;
 	
-	return compare_and_swaplp((atomic_l)papDestination, &liOldValue, (long)apExchange);
+	return compare_and_swaplp((atomic_l)papDestination, &liOldValue, (__ou_caslp_value_t)apExchange);
 }
 
 
 #endif // #if _OU_TARGET_BITS == _OU_TARGET_BITS_64
 
 
+#undef __ou_caslp_value_t
+#undef __ou_cas_value_t
+
+
 #endif // #if _OU_TARGET_OS == _OU_TARGET_OS_AIX
 
 
@@ -1205,9 +1270,15 @@ static _OU_ALWAYSINLINE void _OU_CONVENTION_API
 
 // No atomic functions for generic UNIX
 
-// x86 assembler implementation for i486 must be engaged explicitly
-#if defined(_OU_ATOMIC_USE_X86_ASSEMBLER)
+#if _OU_COMPILER == _OU_COMPILER_GCC && (_OU_TARGET_ARCH == _OU_TARGET_ARCH_X86 || _OU_TARGET_ARCH == _OU_TARGET_ARCH_X64)
 
+#define __OU_ATOMIC_X86_ASSEMBLER_USE_AUTOENABLED    1
+
+
+#endif // Otherwise the x86 assembler implementation must be engaged explicitly
+
+
+#if _OU_ATOMIC_USE_X86_ASSEMBLER || (__OU_ATOMIC_X86_ASSEMBLER_USE_AUTOENABLED && !defined(_OU_ATOMIC_USE_X86_ASSEMBLER))
 
 typedef uint32ou atomicord32;
 typedef void *atomicptr;
@@ -1225,13 +1296,13 @@ struct _ou_atomic_CLargeStruct
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicIncrement(volatile atomicord32 *paoDestination)
 {
-	register atomicord32 aoResult = 1;
+	atomicord32 aoResult = 1;
 
 	asm volatile (
 		"lock; xaddl %2, %0;"
 		: "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination), "=r" (aoResult)
 		: "1" (aoResult), "m" (*paoDestination)
-		: "memory");
+		: "memory", "cc");
 
 	return aoResult + 1;
 }
@@ -1239,13 +1310,13 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicDecrement(volatile atomicord32 *paoDestination)
 {
-	register atomicord32 aoResult = (atomicord32)(-1);
+	atomicord32 aoResult = (atomicord32)(-1);
 
 	asm volatile (
 		"lock; xaddl %2, %0;"
 		: "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination), "=r" (aoResult)
 		: "1" (aoResult), "m" (*paoDestination)
-		: "memory");
+		: "memory", "cc");
 
 	return aoResult - 1;
 }
@@ -1254,10 +1325,10 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicExchange(volatile atomicord32 *paoDestination, atomicord32 aoExchange)
 {
-	register atomicord32 aoResult;
+	atomicord32 aoResult;
 
 	asm volatile (
-		"xchg %2, %0;"
+		"xchgl %2, %0;"
 		: "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination), "=r" (aoResult)
 		: "1" (aoExchange), "m" (*paoDestination)
 		: "memory");
@@ -1268,13 +1339,13 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicExchangeAdd(volatile atomicord32 *paoDestination, atomicord32 aoAddend)
 {
-	register atomicord32 aoResult;
+	atomicord32 aoResult;
 
 	asm volatile (
 		"lock; xaddl %2, %0;"
 		: "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination), "=r" (aoResult)
 		: "1" (aoAddend), "m" (*paoDestination)
-		: "memory");
+		: "memory", "cc");
 
 	return aoResult;
 }
@@ -1282,14 +1353,14 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE bool _OU_CONVENTION_API 
 /*bool */AtomicCompareExchange(volatile atomicord32 *paoDestination, atomicord32 aoComparand, atomicord32 aoExchange)
 {
-	register bool bResult;
+	bool bResult;
 
 	asm volatile (
-		"lock; cmpxchgl %3, %0;"
-		"setzb %1;"
-		: "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination), "=a" (bResult)
+        "lock; cmpxchgl %3, %0;"
+        "setzb %1;"
+        : "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination), "=a" (bResult)
 		: "a" (aoComparand), "r" (aoExchange), "m" (*paoDestination)
-		: "memory");
+		: "memory", "cc");
 
 	return bResult;
 }
@@ -1300,8 +1371,8 @@ static _OU_ALWAYSINLINE bool _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicAnd(volatile atomicord32 *paoDestination, atomicord32 aoBitMask)
 {
-	register atomicord32 aoResult;
-	register atomicord32 aoExchange;
+	atomicord32 aoResult;
+	atomicord32 aoExchange;
 
 	asm volatile (
 	"0:;"
@@ -1311,7 +1382,7 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 		"jnz   0b;"
 		: "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination), "=a" (aoResult), "=&r" (aoExchange)
 		: "a" (*paoDestination), "g" (aoBitMask), "m" (*paoDestination)
-		: "memory");
+		: "memory", "cc");
 
 	return aoResult;
 }
@@ -1319,8 +1390,8 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicOr(volatile atomicord32 *paoDestination, atomicord32 aoBitMask)
 {
-	register atomicord32 aoResult;
-	register atomicord32 aoExchange;
+	atomicord32 aoResult;
+	atomicord32 aoExchange;
 
 	asm volatile (
 	"0:;"
@@ -1330,7 +1401,7 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 		"jnz   0b;"
 		: "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination), "=a" (aoResult), "=&r" (aoExchange)
 		: "a" (*paoDestination), "g" (aoBitMask), "m" (*paoDestination)
-		: "memory");
+		: "memory", "cc");
 
 	return aoResult;
 }
@@ -1338,8 +1409,8 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API 
 /*atomicord32 */AtomicXor(volatile atomicord32 *paoDestination, atomicord32 aoBitMask)
 {
-	register atomicord32 aoResult;
-	register atomicord32 aoExchange;
+	atomicord32 aoResult;
+	atomicord32 aoExchange;
 
 	asm volatile (
 	"0:;"
@@ -1349,12 +1420,107 @@ static _OU_ALWAYSINLINE atomicord32 _OU_CONVENTION_API
 		"jnz   0b;"
 		: "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination), "=a" (aoResult), "=&r" (aoExchange)
 		: "a" (*paoDestination), "g" (aoBitMask), "m" (*paoDestination)
-		: "memory");
+		: "memory", "cc");
 
 	return aoResult;
 }
 
 
+#define __OU_ATOMIC_ORD32_NORESULT_FUNCTIONS_DEFINED
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicIncrementNoResult(volatile atomicord32 *paoDestination)
+{
+    asm volatile (
+        "lock; incl %0;"
+        : "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination)
+        : "m" (*paoDestination)
+        : "memory", "cc");
+}
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicDecrementNoResult(volatile atomicord32 *paoDestination)
+{
+    asm volatile (
+        "lock; decl %0;"
+        : "=m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination)
+        : "m" (*paoDestination)
+        : "memory", "cc");
+}
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicExchangeAddNoResult(volatile atomicord32 *paoDestination, atomicord32 aoAddend)
+{
+    asm volatile (
+        "lock; addl %1, %0;"
+        : "=m,m,m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination)
+        : "r,K,i" (aoAddend), "m,m,m" (*paoDestination)
+        : "memory", "cc");
+}
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicAndNoResult(volatile atomicord32 *paoDestination, atomicord32 aoBitMask)
+{
+    asm volatile (
+        "lock; andl %1, %0;"
+        : "=m,m,m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination) 
+        : "r,K,i" (aoBitMask), "m,m,m" (*paoDestination)
+        : "memory", "cc");
+}
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicOrNoResult(volatile atomicord32 *paoDestination, atomicord32 aoBitMask)
+{
+    asm volatile (
+        "lock; orl %1, %0;"
+        : "=m,m,m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination) 
+        : "r,K,i" (aoBitMask), "m,m,m" (*paoDestination)
+        : "memory", "cc");
+}
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicXorNoResult(volatile atomicord32 *paoDestination, atomicord32 aoBitMask)
+{
+    asm volatile (
+        "lock; xorl %1, %0;"
+        : "=m,m,m" (*(volatile _ou_atomic_CLargeStruct *)paoDestination) 
+        : "r,K,i" (aoBitMask), "m,m,m" (*paoDestination)
+        : "memory", "cc");
+}
+
+
+#define __OU_ATOMIC_PTR_FUNCTIONS_DEFINED
+
+static _OU_ALWAYSINLINE atomicptr _OU_CONVENTION_API 
+/*atomicptr */AtomicExchangePointer(volatile atomicptr *papDestination, atomicptr apExchange)
+{
+    atomicptr apResult;
+
+    asm volatile (
+        "xchg %2, %0;"
+        : "=m" (*(volatile _ou_atomic_CLargeStruct *)papDestination), "=r" (apResult)
+        : "1" (apExchange), "m" (*papDestination)
+        : "memory");
+
+    return apResult;
+}
+
+static _OU_ALWAYSINLINE bool _OU_CONVENTION_API 
+/*bool */AtomicCompareExchangePointer(volatile atomicptr *papDestination, atomicptr apComparand, atomicptr apExchange)
+{
+    bool bResult;
+
+    asm volatile (
+        "lock; cmpxchg %3, %0;"
+        "setzb %1;"
+        : "=m" (*(volatile _ou_atomic_CLargeStruct *)papDestination), "=a" (bResult)
+        : "a" (apComparand), "r" (apExchange), "m" (*papDestination)
+        : "memory", "cc");
+
+    return bResult;
+}
+
+
 #endif // #if defined(_OU_ATOMIC_USE_X86_ASSEMBLER)
 
 
@@ -1509,11 +1675,13 @@ bool _OU_CONVENTION_API AtomicCompareExchangePointer(volatile atomicptr *papDest
 // Doxygen fooling declarations (used for documentation generation only)
 void _OU_CONVENTION_API AtomicIncrementNoResult(volatile atomicord32 *paoDestination);
 void _OU_CONVENTION_API AtomicDecrementNoResult(volatile atomicord32 *paoDestination);
+void _OU_CONVENTION_API AtomicStore(volatile atomicord32 *paoDestination, atomicord32 aoValue);
 void _OU_CONVENTION_API AtomicExchangeAddNoResult(volatile atomicord32 *paoDestination, atomicord32 aoAddend);
 void _OU_CONVENTION_API AtomicAndNoResult(volatile atomicord32 *paoDestination, atomicord32 aoBitMask);
 void _OU_CONVENTION_API AtomicOrNoResult(volatile atomicord32 *paoDestination, atomicord32 aoBitMask);
 void _OU_CONVENTION_API AtomicXorNoResult(volatile atomicord32 *paoDestination, atomicord32 aoBitMask);
-
+void _OU_CONVENTION_API AtomicStorePointer(volatile atomicptr *papDestination, atomicptr apValue);
+void _OU_CONVENTION_API AtomicReadReorderBarrier();
 
 #endif // #if defined(__OU_DOXYGEN__)
 
@@ -1574,6 +1742,68 @@ static _OU_ALWAYSINLINE void _OU_CONVENTION_API
 #endif // #if !defined(__OU_ATOMIC_ORD32_NORESULT_FUNCTIONS_DEFINED)
 
 
+//////////////////////////////////////////////////////////////////////////
+// Default Load/Store function implementations
+
+#if !defined(__OU_ATOMIC_READREORDERBARRIER_FUNCTION_DEFINED) && _OU_COMPILER == _OU_COMPILER_GCC
+
+// These architectures are known (by me ;-))to retain memory read request order
+#if (_OU_TARGET_ARCH == _OU_TARGET_ARCH_X86 || _OU_TARGET_ARCH == _OU_TARGET_ARCH_X64)
+
+#define __OU_ATOMIC_READREORDERBARRIER_FUNCTION_DEFINED
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicReadReorderBarrier()
+{
+    // x86/x64 architectures do not reorder memory reads anyway
+    // it's only necessary to stop compiler from reordering read instructions
+    asm volatile ( "" : : :"memory" );
+}
+
+
+#endif // #if (_OU_TARGET_ARCH == _OU_TARGET_ARCH_X86 || _OU_TARGET_ARCH == _OU_TARGET_ARCH_X64)
+
+
+#endif // #if !defined(__OU_ATOMIC_READREORDERBARRIER_FUNCTION_DEFINED) && _OU_COMPILER == _OU_COMPILER_GCC
+
+
+#ifndef __OU_ATOMIC_READREORDERBARRIER_FUNCTION_DEFINED
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicReadReorderBarrier()
+{
+    atomicord32 aoTemporaryValue;
+    AtomicExchangeAddNoResult(&aoTemporaryValue, 0);
+}
+
+
+#endif // #ifndef __OU_ATOMIC_READREORDERBARRIER_FUNCTION_DEFINED
+
+
+#ifndef __OU_ATOMIC_STORE_FUNCTION_DEFINED
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicStore(volatile atomicord32 *paoDestination, atomicord32 aoValue)
+{
+    AtomicExchange(paoDestination, aoValue);
+}
+
+
+#endif // #ifndef __OU_ATOMIC_STORE_FUNCTION_DEFINED
+
+
+#ifndef __OU_ATOMIC_STOREPTR_FUNCTION_DEFINED
+
+static _OU_ALWAYSINLINE void _OU_CONVENTION_API 
+/*void */AtomicStorePointer(volatile atomicptr *papDestination, atomicptr apValue)
+{
+    AtomicExchangePointer(papDestination, apValue);
+}
+
+
+#endif // #ifndef __OU_ATOMIC_STOREPTR_FUNCTION_DEFINED
+
+
 //////////////////////////////////////////////////////////////////////////
 // Atomic initialization function stubs
 
@@ -1599,5 +1829,7 @@ static _OU_INLINE void _OU_CONVENTION_API FinalizeAtomicAPI()
 END_NAMESPACE_OU();
 
 
+#endif // #if _OU_FEATURE_SET >= _OU_FEATURE_SET_ATOMICS
+
 
 #endif // #ifndef __OU_ATOMIC_H_INCLUDED
