@@ -30,6 +30,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 #include <UnitTest++.h>
 #include <ode/ode.h>
+#include "../ode/src/config.h"
 #include "../ode/src/joints/joints.h"
 
 
@@ -78,6 +79,7 @@ SUITE(JointHinge2)
 
     TEST_FIXTURE(Hinge2GetInfo1_Fixture_1, test_hinge2GetInfo1)
     {
+        /*
         //       ^Y
         //     |---|                             HiStop
         //     |   |                     ^Y         /
@@ -95,6 +97,7 @@ SUITE(JointHinge2)
         //
         //
         //
+        */
         dMatrix3 R;
 
         dJointSetHinge2Param(jId, dParamLoStop, -M_PI/4.0);
@@ -261,6 +264,7 @@ SUITE(JointUniversal)
 
     TEST_FIXTURE(UniversalGetInfo1_Fixture_1, test_hinge2GetInfo1_RotAroundX)
     {
+        /*
         //       ^Y
         //     |---|                             HiStop
         //     |   |                     ^Y         /
@@ -278,6 +282,7 @@ SUITE(JointUniversal)
         //
         //
         //
+        */
         dMatrix3 R;
 
         dJointSetUniversalParam(jId, dParamLoStop, -M_PI/4.0);
@@ -401,6 +406,7 @@ SUITE(JointUniversal)
 
     TEST_FIXTURE(UniversalGetInfo1_Fixture_1, test_hinge2GetInfo1_RotAroundY)
     {
+        /*
         //       ^Y
         //     |---|                             HiStop
         //     |   |                     ^Y         /
@@ -418,6 +424,7 @@ SUITE(JointUniversal)
         //
         //
         //
+        */
         dMatrix3 R;
 
         dJointSetUniversalParam(jId, dParamLoStop, -M_PI/4.0);
